
"""
日志类
1、支持多进程/线程
2、支持日志按日期分割
3、支持日志打印时间前缀
"""

import io
import os
import sys
import time
import datetime
import platform
import colorama # http://pypi.python.org/pypi/colorama
import chardet
import threading
import queue
from . import func
from . import define
# colorama.init(wrap=False)
#from common import web

_log_mode = 'w'
_log_rotating = False
_log_time_prifix = False
_sys_log = None
# log_queue = queue.Queue()
if os.name == 'nt':
    import win32con, win32file, pywintypes # pip install pypiwin32
    LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
    LOCK_SH = 0  # The default value
    LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
    __overlapped = pywintypes.OVERLAPPED()

    def lock(file, flags):
        hfile = win32file._get_osfhandle(file.fileno())
        win32file.LockFileEx(hfile, flags, 0, 0xffff0000, __overlapped)

    def unlock(file):
        hfile = win32file._get_osfhandle(file.fileno())
        win32file.UnlockFileEx(hfile, 0, 0xffff0000, __overlapped)

elif os.name == 'posix':
    import fcntl
    from fcntl import LOCK_EX
    def lock(file, flags):
        fcntl.flock(file.fileno(), flags)

    def unlock(file):
        fcntl.flock(file.fileno(), fcntl.LOCK_UN)
else:
    raise RuntimeError("File Locker only support NT and Posix platforms!")

#class class_thread_print_log(threading.Thread):
#    def __init__(self, log_queue):
#        threading.Thread.__init__(self)
#        self.log_queue = log_queue

#    def run(self):
#        while True:
#            if not self.log_queue.empty():
#                file_name, content = self.log_queue.get()
#                _HandleLog(file_name, content)
#            #else:
#                self.log_queue.task_done()

def set_log_mode(mode:str):
    """设置日志的文件写模式，通常为w或者a"""
    global _log_mode
    _log_mode = mode

def set_log_rotating(value:bool):
    """设置日志是否分割，如果分割则日志名称加当前日期，否则不加"""
    global _log_rotating
    _log_rotating = value
    
def set_log_time_prifix(value:bool):
    """设置日志是否要时间前缀"""
    global _log_time_prifix
    _log_time_prifix = value

class class_log:
    enable = True
    clr_inited = False

    clr_blue =  colorama.Style.BRIGHT + colorama.Fore.BLUE
    clr_green =  colorama.Style.BRIGHT + colorama.Fore.GREEN
    clr_red =  colorama.Style.BRIGHT + colorama.Fore.RED
    clr_yellow =  colorama.Style.BRIGHT + colorama.Fore.YELLOW
    clr_mag = colorama.Style.BRIGHT + colorama.Fore.MAGENTA

    def __init__(self, path):
        self._file_path = path
        self._file = open(path,_log_mode)
        self._fil = sys.stdout
        if hasattr(self._fil, "isatty") and self._fil.isatty() and colorama:
            self._fil = colorama.AnsiToWin32(self._fil).stream

        # if not self.clr_inited :
        #     colorama.init(autoreset=True)

        self.isWindows = ('win32' == sys.platform)

    def __del__(self):
        # if self.clr_inited:
        #     colorama.deinit()
        self.close()

    def _write_file(self, c):
        while True:
            try: # 已经改为及时锁unblock模式，还需要进一步测试可靠性
                lock(self._file, LOCK_EX|LOCK_NB)
                break
            except pywintypes.error as e:
                time.sleep(0.01)
                continue
            except Exception as e:
                with open('log_error.txt', 'a') as f:
                    time_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f ")
                    f.write('%s %s,%s\n' % (time_prefix, e, c))
                break
        try:
            #stime = time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(time.time()))
            c = '%s\n' % c
            # c = c.encode("gbk")
            self._file.write(c)
            self._file.flush()
        except:
            pass

        while True:
            try:
                unlock(self._file)
                return True
            except:
                time.sleep(0.01)
                continue

    def write_console(self, c):
        # if self.isWindows : c = c.encode('utf8', errors='ignore')
        # else : c = c.encode('utf8')
        # if not sys.stdout.encoding: c = c.encode('utf8')
        type = sys.getfilesystemencoding()
        # c = c.encode(type)

        # pycharm can do
        # sys.stdout.write(c)

        # console can do
        self._fil.write(c)



    def write(self, clr, datetime, c):
        self._write_file(datetime + c)

        if self.enable :
            c = self.clr_yellow + datetime + clr + c + colorama.Style.RESET_ALL + '\n'
            self.write_console(c)

    def write_trace(self, c):
        strg = func.format_traceback()
        if not strg:
            self.write(self.clr_red, 'trace lost.%s'%c)
            return

        self.write(self.clr_red, strg)
        self.write(self.clr_red, c)

    def close(self):
        self._file.close()


# 设置日志是否显示在控制台
_log_console_enable = {}
def SetConsoleEnable(file_name, enable):
    _log_console_enable[file_name] = enable

_log_files = {}
def close_all():
    global _log_files
    for k,v in list(_log_files.items()):
        v.close()
    _log_files = {}
    
    global _sys_log
    if _sys_log:
        _sys_log = None

def WriteLog(file_name, content):
    _HandleLog(file_name, content)
    #log_queue.put((file_name, content))

def WriteData(file_name, content):
    """纯数据类型，不需要时间前缀"""
    _HandleLog(file_name, content, prefix=False)

# 写日志，如果没有日志对象则创建
def _HandleLog(file_name, content, prefix=True):
    global _log_files
    global _log_time_prifix
    global _log_rotating

    if file_name in define.g_log_switch and not define.g_log_switch[file_name]:
        return

    if not isinstance(content, str) and not isinstance(content, str):
        content = content.__str__()

    # 创建log目录
    base_path = os.getcwd()
    base_path = os.path.dirname(os.path.realpath(sys.argv[0]))
    root = os.path.join(base_path, "log")
    
    if _log_rotating:
        root = os.path.join(root, datetime.datetime.now().strftime("%Y-%m"))
        date = datetime.datetime.now().strftime("%m-%d")
        path = "%s/%s %s.log" % (root, file_name, date)
    else:
        path = "%s/%s.log" % (root, file_name)

    if not os.path.exists(root): os.makedirs(root)
    if path not in _log_files: _log_files[path] = class_log(path)

    if not isinstance(content, str):
        if chardet.detect(content)["encoding"] == "utf-8":
            content = content.decode("utf-8")
        elif chardet.detect(content)["encoding"] == "GB2312":
            content = content.decode("GB2312")

    try :
        if class_log.enable:
            _log_files[path].write_console("%s: " % file_name)

        time_prefix = ""
        if _log_time_prifix and prefix:
            time_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f\t")
        _log_files[path].write(class_log.clr_green, time_prefix, content)
    except Exception as e:
        print(e)

##########################################################################################
# 系统日志

def _Get_sys_log():
    global _sys_log
    if not _sys_log: _sys_log = class_log("log/sys.log")
    return _sys_log

def WriteError(strg):
    try :
        time_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f ")
        _Get_sys_log().write(class_log.clr_mag, time_prefix, strg)
    except Exception as e:
        print(e)

def WriteWarning(strg):
    try :
        _Get_sys_log().write(class_log.clr_mag, strg)
    except : pass

def WriteExcept(strg):
    try :
        _Get_sys_log().write_trace(strg)

    except Exception as e:
        print(e)
    # raise exception.
    # don't put it into try block
    sys.exit()

def WriteBlock(strg):
    _Get_sys_log().write(class_log.clr_blue, '======= %s ========'%strg)

def WriteEntry(strg):
    _Get_sys_log().write(class_log.clr_green, '▶ %s'%strg)

def WriteDetail(strg):
    _Get_sys_log().write(class_log.clr_yellow, ' - %s'%strg)

def WriteRaw(strg):
    if not _Get_sys_log().enable:return
    sys.stdout.write(strg)

def Disable(yes = True):
    _Get_sys_log().enable = not yes




import sys,os

#__file__获取执行文件相对路径，整行为取上一级目录
BASE_DIR=os.path.dirname(os.path.abspath(__file__))

# 添加路径
sys.path.append(BASE_DIR)

# 修改当前工作目录
os.chdir(BASE_DIR)

# 获取当前工作目录
# abcdir = os.getcwd()
# print(abcdir)


# -*- coding: utf-8 -*-
# @Time    : 2017/11/3 15:38
# @Author  : hc
# @Site    :
# @File    : _TradeX.py
# @Software: PyCharm
import ctypes
import pandas as pd
import common.pyetc as pyetc
import datetime
import os
class _TradeX():
    def __init__(self,account_id):
        self.max_num = 60 # 查询历史数据的周期
        self.conf_path = r'..\..\conf\actStock.py'
        self.dll_path = r'.\dll\trade_%s.dll'
        self.account_conf = pyetc.load(self.conf_path).ACCOUNT_INFO[account_id]
        self.dll = self.load_dll()

        self.ClientID = -1
        self.OpenTdx()
        self.ErrInfo = ctypes.create_string_buffer('/0' * 256)

    def load_dll(self):
        dll_filename = self.dll_path%self.account_conf["AccountNo"]
        if os.path.exists(dll_filename):
            return ctypes.windll.LoadLibrary(dll_filename)
        if self.gen_dll():
            return ctypes.windll.LoadLibrary(dll_filename)
        return None
    def gen_dll(self):
        res = self.gen_keyword()
        f = open(self.dll_path%"CCHOGIBI", 'rb')
        file_data = f.read()
        f.close()
        # print filedata[0x1148A8:0x1148B7]
        file_data2 = file_data[:0x1148A9]
        for i in res:
            file_data2+=i
        for cnt in range(15-len(res)):
            file_data2+=chr(00)
        file_data2+=file_data[0x1148B8:]
        f2 = open(self.dll_path%self.account_conf["AccountNo"], 'wb')
        f2.write(file_data2)
        f2.close()
        print("破解文件已完成%s"%(self.dll_path%self.account_conf["AccountNo"]))
        return True
    def gen_keyword(self):
        account_id = self.account_conf["AccountNo"]
        res_account = ""  # 账号的奇数账号
        print("准备生成破解文件。账号：%s 奇数位账号：%s"%(account_id,res_account))
        for i in range(len(account_id)):
            if i % 2 == 0:
                res_account += account_id[i]
        a3 = ctypes.c_uint16()
        a3.value = 0x55E
        result = ""
        for i in res_account:
            a = ord(i)
            b = a3.value >> 0x8
            c = a ^ b
            a3.value = 0x207F * (a3.value + c) - 0x523D
            next_flat = True
            for j in range(65, 91):
                if next_flat:
                    for k in range(90, 64, -1):
                        tmp = 1755 + c - k
                        # print("i:[%s] j[%d] k[%d] a[%s] b[%d] c[%d] a3[%s] tmp[%s]" % (i, j, k, a, b, c, a3, tmp))
                        if (tmp % 26 == 0 and tmp / 26 == j):
                            result += chr(j) + chr(k)
                            next_flat = False
                            break
                else:
                    break
        return result
    def OpenTdx(self):
        return self.dll.OpenTdx()
    def CloseTdx(self):
        return self.dll.CloseTdx()

    def __del__(self):
        # pass
        self.Logoff()
        self.CloseTdx()
    def Logoff(self):
        # Result = ctypes.create_string_buffer('/0' * 1024)
        if self.ClientID!=-1:
            self.dll.Logoff(self.ClientID)
    def Logon(self):
        self.ClientID = self.dll.Logon(self.account_conf["IP"], self.account_conf["Port"], self.account_conf["Version"], self.account_conf["YybID"], self.account_conf["AccountNo"], self.account_conf["TradeAccount"], self.account_conf["JyPassword"], self.account_conf["TxPassword"], self.ErrInfo)
        if self.ClientID == -1:
            print(self.ErrInfo.value.decode("gbk"))
        return self.ClientID

    def _QueryData(self,nCategory):
        Result = ctypes.create_string_buffer('/0' * 1024)
        self.dll.QueryData(self.ClientID, nCategory, Result, self.ErrInfo)  # 查询资金
        if self.ErrInfo.value:
            print(self.ErrInfo.value.decode("gbk"))
            return None
        return Result.value.decode("gbk")

    # nCategory:查询信息的种类，
    # 0:资金
    # 1:股份
    # 2:当日委托
    # 3:当日成交
    # 4:可撤单
    # 5:股东代码
    # ====以下未调试成功
    # 6:融资余额
    # 7:融券余额
    # 8:可融证券
    # 9
    # 10
    # 11
    # 12:可申购新股查询
    # 13:新股申购额度查询
    # 14:配号查询
    # 15:中签查询
    def QueryData(self,nCategory=0):
        res = self._QueryData(nCategory)
        if res:
            res = [_r.split("\t") for _r in  res.split("\n")]
            df = pd.DataFrame(res[1:],columns=res[0])
            return df

    def _QueryHistoryData(self,nCategory, sStartDate, sEndDate):
        sStartDate = str(sStartDate)
        sEndDate = str(sEndDate)
        Result = ctypes.create_string_buffer('/0' * (1024*10240))
        self.dll.QueryHistoryData(self.ClientID,nCategory,sStartDate,sEndDate,Result,self.ErrInfo)
        if self.ErrInfo.value:
            print(self.ErrInfo.value.decode("gbk"))
            return None
        return Result.value.decode("gbk")

    def gen_date_list(self,sStartDate,sEndDate):
        date_list = []
        startDate = datetime.datetime.strptime(sStartDate, '%Y%m%d')
        endDate = datetime.datetime.strptime(sEndDate, '%Y%m%d')
        delta = endDate - startDate
        range_num = delta.days // self.max_num + 1
        _startDate = 0
        for i in range(range_num):
            dt = datetime.timedelta(days=self.max_num)
            _startDate = startDate if not _startDate else _startDate + dt
            _endDate = endDate if i == range_num - 1 else _startDate + dt - datetime.timedelta(days=1)
            date_list.append((_startDate.strftime('%Y%m%d'), _endDate.strftime('%Y%m%d')))
        return date_list
    # nCategory:查询信息的种类，
    # 0:历史委托
    # 1:历史成交
    # 2:交割单
    def QueryHistoryData(self,nCategory,sStartDate,sEndDate):
        # """
        df= pd.DataFrame()
        date_list = self.gen_date_list(sStartDate,sEndDate)
        for _date in date_list:
            res = self._QueryHistoryData(nCategory, _date[0], _date[1])
            if res:
                res = [_r.split("\t") for _r in res.split("\n")]
                # if df.empty:
                # df = pd.DataFrame(res[1:], columns=res[0])

                _df = pd.DataFrame(res[1:], columns=res[0])
                # print("==_df==",_date,_df)
                df = df.append(_df,ignore_index=True)

        return df
        # """

if __name__ == "__main__":
    a = _TradeX("305600050220")
    a.Logon()
    for i in range(0,6):
        r = a.QueryData(i)
        print(r)

    # r = a.QueryHistoryData(1,"20170111","20171111")

    # print(a.ClientID)
    # print(r)






# -*- coding: utf-8 -*-
from tradeTools import helpers
from tradeTools import Decorator
from tmqc.frame import data_center
from tmqc.frame import stock_func
from datetime import datetime
import pandas as pd
import numpy as np
import os

QUERY_CONF = {} # 查询配置
QUERY_CONF["PE"] = {}
QUERY_CONF["PE"]["income"] = {}
QUERY_CONF["PE"]["income"]["npParentCompanyOwners"] = "归属于母公司股东的净利润"
    
class Mgr():
    def __init__(self):
        self.oldDc = data_center.use()

    def queryST(self, dateTime = 20210101):
        # 用连续两年的年报日期的数据来判断ST
        reportYear = dateTime // 10000 - 1 if dateTime % 10000 >= 430 else dateTime // 10000 - 2
        queryConf = {}
        queryConf["RecurringNetProfit"] = "扣除非经常性损益后的净利润"
        reportDate = reportYear * 10000 + 1231
        lastReportDate = (reportYear - 1) * 10000 + 1231
        sql = "SELECT "
        for key in queryConf.keys():
            sql += key + ","
        sql += f"symbol,reportDate FROM `income` WHERE reportDate in ({reportDate},{lastReportDate}) "
        df = pd.read_sql(sql, self.oldDc.database.conn)
        df = df.set_index("symbol")
        stSymbols = []
        for symbol in set(df.index.tolist()):
            _df = df.loc[symbol, ["RecurringNetProfit"]]
            __df = _df[_df.values > 0]  # 两年全负数，
            if __df.empty:
                stSymbols.append(symbol)
        return stSymbols
    
    # 获取上市满1年的股票
    # 过滤st股
    def getSymbols(self,dateTime = 20210101,isRealTime = False):
        if not hasattr(self, 'listDf'):
            self.listDf = helpers.getSymbolsInfo(isRealTime = isRealTime)
            self.listDf = self.listDf.drop_duplicates(subset=["symbol",],keep='first')
            self.listDf = self.listDf.set_index("symbol")
            listDf = self.listDf
        else:
            listDf = self.listDf
        start = datetime.strptime(str(dateTime), '%Y%m%d') + pd.tseries.offsets.DateOffset(years = -1)
        symbols= listDf[listDf.listingDate<=start].index.tolist()
        symbols = list(set(symbols) - set(self.queryST(dateTime =dateTime)))
        return symbols
    
    @Decorator.loadData(path="data")
    def genIDXData(self, indexSymbol,isRealTime = False,**kwargs):
        # 获取指数月度收益率数据
        # fileExtension ：间隔日
        if not hasattr(self, "oldDc"):
            self.oldDc = data_center.use()
        maxDate = 0
        fileName = kwargs["fileName"]
        if  os.path.exists(fileName):
            oldDf = pd.read_excel(fileName, sheet_name="数据源", index_col=0,engine="openpyxl")
            maxDate = oldDf["time"].max()
        sql = "SELECT "
        sql += f" code,time,close from `index_day_data` WHERE code = '{indexSymbol}'"
        if maxDate:
            sql+=f" and time>{maxDate}"
        df = pd.read_sql(sql, self.oldDc.database.conn)
        if df.empty:
            return oldDf
        
        df['date'] = pd.to_datetime(df['time'], format='%Y%m%d')
        df.set_index('date', inplace=True)

        if maxDate:
            df = oldDf.append(df)
        df[indexSymbol] = df.close / df.close.shift(1) - 1
        return df

    @Decorator.loadData(path="data")
    def genSymbolRateData(self, symbol, frequency, isRealTime=False,**kwargs):
        # 生成间隔数据收益率数据
        if not hasattr(self, "oldDc"):
            self.oldDc = data_center.use()
        maxDate = 0
        fileName = kwargs["fileName"]
        if  os.path.exists(fileName): # 读取已有数据文件
            oldDf = pd.read_excel(fileName, sheet_name="数据源", index_col=0,engine="openpyxl")
            if not oldDf.empty:
                maxDate = oldDf["time"].max()
            
        sql = "SELECT "
        sql += f" code,time,close,volume from `tdx_day_data` WHERE code = '{symbol}'"
        if maxDate:
            sql+=f" and time >={maxDate}" # 多取一条历史数据
        df = pd.read_sql(sql, self.oldDc.database.conn)
        if df.empty:
            if maxDate:
                return oldDf
            return pd.DataFrame()

        df['date'] = pd.to_datetime(df['time'], format='%Y%m%d')
        df.set_index('date', inplace=True)
        
        df["lastTime"] = df.time.shift()
        df["lastClose"] = df.close.shift()

        def _getRehabilitationClose(hang):  # 复权
            begTime = hang.lastTime
            endTime = hang.time
            symbol = hang.code
            dividends = self.oldDc.query_dividends(stock_id=symbol, begtime=begTime, endtime=endTime)
            pre_close = hang.lastClose
            if dividends[symbol]:
                for dividend in dividends[symbol]:
                    pre_close = stock_func.get_dividend_pre_price(pre_close, dividend[1])  # 前复权
            return (hang.close / pre_close) - 1

        # 如果交易日停牌。则当日和下一个间隔交易日收益率为空
        df["rate"] = df.apply(_getRehabilitationClose, axis=1)
        if maxDate:
            df = df[df.time>maxDate]
            df = oldDf.append(df)
        return df
    
        # sql = "SELECT "
        # sql += f" id,datetime,total_capital as totalCapital,unlimited_sell_shares as unlimitedSellShares from `std_dzh_capital_report` WHERE id = '{symbol}'"
        # capital = pd.read_sql(sql, self.oldDc.database.conn)
        # capital['date'] = pd.to_datetime(capital['datetime'], format='%Y%m%d')
        # capital.set_index('date', inplace=True)
        # capital = capital[["totalCapital","unlimitedSellShares"]]/10**4
        # conDf = pd.concat([df, capital], join="outer", axis=1)
        # conDf["totalCapital"].fillna(method='ffill', inplace=True)  # ’ffill’，向前填充，或是向下填充
        # conDf["unlimitedSellShares"].fillna(method='ffill', inplace=True)  # ’ffill’，向前填充，或是向下填充

        # conDf["MarketV"] = conDf["close"]*conDf["total_capital"]
        # conDf["MarketV"].fillna(method='ffill', inplace=True)  # ’ffill’，向前填充，或是向下填充
        # conDf["TurnOver"] = conDf["volume"]*100/conDf["unlimited_sell_shares"]*100
        # conDf = conDf.dropna(axis=0,subset = ["交易日"]) 
        # return conDf

    @Decorator.loadData(path="data")
    def genSymbolTScore(self, symbol, frequency,windows,):#滚动z分数
        allData = self.genSymbolRateData(symbol=symbol,frequency=frequency)
        if allData.empty:
            return pd.DataFrame()
        allData['mRate'].fillna(0, inplace=True) # 停牌的日期。收益率为设为0
        allData['mean'] = allData['mRate'].rolling(windows).mean()
        allData['std'] = allData['mRate'].rolling(windows).std(ddof = 1)
        allData["ZScore"] =  (allData['mRate'] - allData['mean'])/allData['std']
        allData["TScore"] =  50+10*allData["ZScore"]
        return allData[['mRate','mean','std','ZScore','TScore']]

    @Decorator.loadData(path="data")
    def genTradeDays(self,frequency :int = 20,isRealTime = False):
        # 根据换仓频率生成交易日列表
        # fileExtension 交易日间隔
        allTradeDays =  self.oldDc.trade_days
        # 切割交易日
        tradeDays = []
        for i in range(len(allTradeDays)//frequency+1):
            index =  (i+1)*frequency-1
            if index < len(allTradeDays):
                tradeDays.append(allTradeDays[index])
        df = pd.DataFrame(tradeDays,columns=["交易日"])
        df['date'] = pd.to_datetime(df['交易日'], format='%Y%m%d')
        df.set_index('date', inplace=True)
        return df

    def getSymbolRateData(self,symbol, frequency=1,isRealTime=False):
        if not hasattr(self, 'symbolRateData'):
                self.symbolRateData = {}
        if symbol not in self.symbolRateData:
            symbolRateData = self.genSymbolRateData(symbol=symbol, frequency=1,
                                                isRealTime=isRealTime)
            if symbolRateData.empty:
                print(f"{symbol}无历史行情数据")
            self.symbolRateData[symbol] = symbolRateData
        return self.symbolRateData.get(symbol)
    
    def getSymbolsData(self,symbols,endDate):
        rates = []
        marketValues = []
        for symbol  in symbols:
            rateDf = self.getSymbolRateData(symbol=symbol,frequency=1)
            star = endDate.replace(endDate.year - 2)
            df = rateDf[(star<=rateDf.index)&(rateDf.index<=endDate)]
            rate = df["mRate"]
            marketValue = df["MarketV"]
            rate.name = symbol
            marketValue.name = symbol
            rates.append(pd.DataFrame(rate))
            marketValues.append(pd.DataFrame(marketValue))
        symbolsReturn = pd.concat(rates,axis=1) 
        marketValue = pd.concat(marketValues,axis=1) 
        marketValue["TOTAL"] = marketValue.sum(axis=1) 
        marketValue = (marketValue.T / marketValue.TOTAL).T # 求权重
        marketValue = marketValue.iloc[-1]
        marketValue = marketValue.drop("TOTAL")
        return symbolsReturn,marketValue  
    
    def genReturn(self,beg,end,symbols,weight = []):
        rate = []
        for symbol in symbols:
            df = self.getSymbolRateData(symbol=symbol,frequency=1)
            # 计算累计收益率
            s = df.loc[(beg<df.index) & (df.index<=end)]["mRate"] 
  
            s.name = symbol
            rate.append(s)
        rateDf = pd.concat(rate,axis=1)
        rateDf = rateDf.fillna(0)  # 停牌股。当日的收益率为0
        # 如果多个交易日。由初始交易日的权重和前一日的净值。得出前一日的权重比例。再和当日的收益率点积
        if rateDf.shape[0] <= 1:
            rate = np.dot(rateDf.iloc[-1],weight)  
        else :
            cumprodDf  =(1 + rateDf).cumprod()
            last = cumprodDf.iloc[-1]
            # 考虑如果总仓位累加不为1
            _p = 1- np.sum(weight) # 初始留存的净值
            rate = _p+np.dot(cumprodDf.iloc[-1],weight) # 当前净值
            rate/= _p+ np.dot(cumprodDf.iloc[-2],weight)# 上一日净值    
            rate -= 1       
        return rate
    
    # 计算symbols组合的收益率
    def genReturnMonthlys(self, dateTime, symbols, frequency, ):
        seriseMonthRate = []
        for symbol in symbols:
            if not hasattr(self, 'symbolRateData'):
                self.symbolRateData = {}

            if symbol not in self.symbolRateData:
                symbolRateData = self.genSymbolRateData(symbol=symbol, frequency=frequency,
                                                        isRealTime=False)
                if symbolRateData.empty:
                    continue
                symbolRateData["lastMarketV"] = symbolRateData["MarketV"].shift()
                symbolRateData['lastMarketV'].fillna(method='ffill', inplace=True)  # ’ffill’，向前填充，或是向下填充
                self.symbolRateData[symbol] = symbolRateData
            df = self.symbolRateData.get(symbol)

            if dateTime not in df.index:
                print(f"{symbol} {dateTime}的数据为空 ")
                continue
            s = df.loc[dateTime]
            seriseMonthRate.append(s)
        monthDf = pd.DataFrame(seriseMonthRate)
        if monthDf.empty:
            return monthDf
        monthDf["市值权重"] = monthDf.lastMarketV / monthDf.lastMarketV.sum()
        monthDf["wRate"] = monthDf.mRate * monthDf["市值权重"]  # 加权收益率
        print(monthDf)
        sumWRate = monthDf.wRate.sum()
        meanRate = monthDf.mRate.mean()
        returnMonthlyDf = pd.DataFrame([[sumWRate, meanRate]], index=[dateTime], columns=['sumWRate', "meanRate"])
        return returnMonthlyDf

    def analyseReturn(self,df):
        tradays = df.shape[0]
        yearRtn = pow(df["cumprod"][-1],252/tradays)-1
        print("复合年化收益",yearRtn)
        r = self.calcSharpRatio(df = df) # 夏普率
        print("最大回撤",df["maximumDrawdown"].max())
        print("最大回撤日期",df["maximumDrawdown"].idxmax())
        print("近半年最大回撤",df["maximumDrawdown"][-120:].max())
        print("近半年最大回撤日期",df["maximumDrawdown"][-120:].idxmax())
        return yearRtn
    
    def calcRtnByYear(self,df):
        s = {}
        for i ,_df in df.resample("Y"):
            _s = (1+_df["return"]).cumprod()
            s[i.year] = _s[-1] - 1 # 每年收益率
        return pd.Series(s)
            
    def calcSharpRatio(self,df):
        # 无风险年化收益率为2%
        # 除非无风险利率波动较大（如在新兴市场中一样），否则超额收益和原始收益的标准差将相似
  
        r = round(df['return'].mean()/df['return'].std()*np.sqrt(252),3)
        print("夏普率",r)
        # 减去无风险利率
        df["rtn"] = df["return"] - 0.02/252
        # 由日频率转化为年化夏普
        # https://www.zhihu.com/question/27264526 不同周期选择参考优劣参考
        r = round(df['rtn'].mean()/df['rtn'].std()*np.sqrt(252),3)
        print("夏普率扣除无风险收益后",r)
        

    @Decorator.loadData()
    def qryReportData(self, symbol,queryConf = "PE",**kawgrs):
        sql = "SELECT "
        tableName = "income"
        queryConf = QUERY_CONF[queryConf][tableName]
        for key in queryConf.keys():
            sql += key + ","
        sql += f"reportDate FROM {tableName} WHERE symbol ='{symbol}'"
        df = pd.read_sql(sql, self.oldDc.database.conn)
        for k, v in queryConf.items():  # 修改列名为【中文+报告期】格式
            df.rename(columns={k: v}, inplace=True)
        df = df.set_index("reportDate")
        df = df.fillna(0)
        return df
    
    def genTTmPE(self,symbol,dateTime:int):
        reportDate = helpers.getReportDate(dateTime=dateTime,isUseYear=False) # 真实可获取的年报日期
        ttmreport = helpers.getTTmReportDates(reportDate) # 上年同季度日期，上年的年报日期，当季报告期
        reportData = self.qryReportData(symbol = symbol,queryConf="PE")
        net = reportData.loc[ttmreport[1]]["归属于母公司股东的净利润"]
        net -= reportData.loc[ttmreport[0]]["归属于母公司股东的净利润"]
        net += reportData.loc[ttmreport[2]]["归属于母公司股东的净利润"]

        print(net)
        close,_ = self.oldDc.get_nearest_price(stock_id = symbol, datetime=dateTime)
        print(close,_)
        capital= self.oldDc.get_capital(datetime=dateTime, stock_id=symbol,factor_name="total_capital")
        print(capital)
        ttmPE = close*capital/net
        return ttmPE,close*capital,net
if __name__ == '__main__':
    m = Mgr()
    CON_FREQUENCY = 1
    CON_INDEX_SYMBOL = "SH.000001"
    # idxDf =  m.genIDXData(indexSymbol= CON_INDEX_SYMBOL,isRealTime=True)
    # print(idxDf)

    # symbolTScore = m.genSymbolTScore(symbol="SH.600000", frequency =1,windows =30)
    # r = m.genSymbolRateData(symbol = "SZ.000549", frequency =1, isRealTime=True)
    # print(r)
    # print(m.qryReportData(symbol = "SZ.000001",queryConf="PE"))
    # print(m.genTTmPE(symbol="SH.000001",dateTime=20211201))
    
    # r = m.genReturn(beg =pd.to_datetime(20070521, format='%Y%m%d'),
    #                 end =pd.to_datetime(20070522, format='%Y%m%d'),
    #                 symbols=["SH.600000","SH.600019"],weight=[0.5,0.5,])
  


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tradeApi.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor # pip install grpcio-tools
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tradeApi.proto',
  package='Trade',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x0etradeApi.proto\x12\x05Trade\"@\n\tRspHeader\x12!\n\x08ret_type\x18\x01 \x01(\x0e\x32\x0f.Trade.RET_TYPE\x12\x10\n\x08sErrInfo\x18\x02 \x01(\t\"\x1b\n\x0bJSONRequest\x12\x0c\n\x04sReq\x18\x01 \x01(\t\"A\n\x0cJSONResponse\x12#\n\trspHeader\x18\x01 \x01(\x0b\x32\x10.Trade.RspHeader\x12\x0c\n\x04sRsp\x18\x02 \x01(\t\"\x8d\x01\n\x07PosItem\x12\x10\n\x08sStkCode\x18\x01 \x01(\t\x12\x1a\n\x06\x62sFlat\x18\x02 \x01(\x0e\x32\n.Trade.DIR\x12\x12\n\nfHoldPrice\x18\x03 \x01(\x02\x12\x0c\n\x04lQty\x18\x04 \x01(\x04\x12\x12\n\nlFrozenQty\x18\x05 \x01(\x04\x12\x0c\n\x04\x66\x41mt\x18\x06 \x01(\x02\x12\x10\n\x08sStkName\x18\x07 \x01(\t\"S\n\x0bRspPosition\x12#\n\trspHeader\x18\x01 \x01(\x0b\x32\x10.Trade.RspHeader\x12\x1f\n\x07posItem\x18\x02 \x03(\x0b\x32\x0e.Trade.PosItem\"\xdf\x02\n\tOrderItem\x12\x10\n\x08sOrderId\x18\x01 \x01(\t\x12\x10\n\x08sStkName\x18\x02 \x01(\t\x12\x12\n\nlOrderTime\x18\x03 \x01(\x04\x12\x1a\n\x06\x62sFlat\x18\x04 \x01(\x0e\x32\n.Trade.DIR\x12\x13\n\x0b\x66OrderPrice\x18\x05 \x01(\x02\x12\x11\n\tlOrderQty\x18\x06 \x01(\x04\x12\x13\n\x0blMatchedQty\x18\x07 \x01(\x04\x12\x15\n\rlWithdrawnQty\x18\x08 \x01(\x04\x12\x10\n\x08sStkCode\x18\t \x01(\t\x12\r\n\x05iDate\x18\n \x01(\x05\x12\'\n\nstatusInfo\x18\x0b \x01(\x0e\x32\x13.Trade.ORDER_STATUS\x12&\n\ncancelFlag\x18\x0c \x01(\x0e\x32\x12.Trade.CANCEL_FLAG\x12\x10\n\x08iErrCode\x18\r \x01(\x05\x12\x0f\n\x07sErrMsg\x18\x0e \x01(\t\x12\x15\n\rfMatchedPrice\x18\x0f \x01(\x02\"i\n\tRspOrders\x12#\n\trspHeader\x18\x01 \x01(\x0b\x32\x10.Trade.RspHeader\x12#\n\torderItem\x18\x02 \x03(\x0b\x32\x10.Trade.OrderItem\x12\x12\n\niTotalSize\x18\x03 \x01(\r\"/\n\x08ReqQuery\x12\x10\n\x08iPageNum\x18\x01 \x01(\r\x12\x11\n\tiPageSize\x18\x02 \x01(\r\"\x95\x02\n\tTransItem\x12\x10\n\x08sOrderId\x18\x01 \x01(\t\x12\x14\n\x0csMatchedTime\x18\x02 \x01(\t\x12\x13\n\x0b\x66MatchedAmt\x18\x03 \x01(\x02\x12\x1a\n\x06\x62sFlat\x18\x04 \x01(\x0e\x32\n.Trade.DIR\x12\x15\n\rfMatchedPrice\x18\x05 \x01(\x02\x12\x13\n\x0blMatchedQty\x18\x06 \x01(\x04\x12\x12\n\nsMatchedSn\x18\x07 \x01(\t\x12\x18\n\x10\x66TotalCommission\x18\x08 \x01(\x02\x12\x31\n\x0ftradeReportType\x18\t \x01(\x0e\x32\x18.Trade.TRADE_REPORT_TYPE\x12\x10\n\x08sStkCode\x18\n \x01(\t\x12\x10\n\x08sStkName\x18\x0b \x01(\t\"T\n\x08RspTrans\x12#\n\trspHeader\x18\x01 \x01(\x0b\x32\x10.Trade.RspHeader\x12#\n\ttransItem\x18\x02 \x03(\x0b\x32\x10.Trade.TransItem\"\x87\x01\n\x0cReqSentOrder\x12\r\n\x05sCode\x18\x01 \x01(\t\x12\x1a\n\x06\x62sFlat\x18\x02 \x01(\x0e\x32\n.Trade.DIR\x12$\n\torderType\x18\x03 \x01(\x0e\x32\x11.Trade.ORDER_TYPE\x12\x13\n\x0biOrderPrice\x18\x04 \x01(\r\x12\x11\n\tlOrderQty\x18\x05 \x01(\x04\"E\n\x0cRspSentOrder\x12#\n\trspHeader\x18\x01 \x01(\x0b\x32\x10.Trade.RspHeader\x12\x10\n\x08sOrderId\x18\x02 \x01(\t\"3\n\x0eReqCancelOrder\x12\x10\n\x08sOrderId\x18\x01 \x01(\t\x12\x0f\n\x07sMarket\x18\x02 \x01(\t\"5\n\x0eRspCancelOrder\x12#\n\trspHeader\x18\x01 \x01(\x0b\x32\x10.Trade.RspHeader*H\n\x08RET_TYPE\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0e\n\nERR_NORMAL\x10\x01\x12\x0f\n\x0b\x45RR_SERIOUS\x10\x02\x12\x0e\n\nERR_IGNORE\x10\x03*#\n\nORDER_TYPE\x12\t\n\x05LIMIT\x10\x00\x12\n\n\x06MARKET\x10\x01*q\n\x0cORDER_STATUS\x12\x0b\n\x07PENDING\x10\x00\x12\n\n\x06TRADED\x10\x01\x12\x0f\n\x0bTRADED_PART\x10\x02\x12\x0e\n\nCANCEL_REQ\x10\x03\x12\x0c\n\x08\x43\x41NCELED\x10\x04\x12\x0c\n\x08REJECTED\x10\x05\x12\x0b\n\x07\x46\x41ILURE\x10\x06*6\n\x11TRADE_REPORT_TYPE\x12\n\n\x06NORMAL\x10\x00\x12\n\n\x06\x43\x41NCEL\x10\x01\x12\t\n\x05WASTE\x10\x02*,\n\x0b\x43\x41NCEL_FLAG\x12\x0e\n\nNOT_CANCEL\x10\x00\x12\r\n\tIS_CANCEL\x10\x01*,\n\x03\x44IR\x12\x07\n\x03\x42UY\x10\x00\x12\x08\n\x04SELL\x10\x01\x12\x08\n\x04\x42OTH\x10\x02\x12\x08\n\x04NULL\x10\x03\x32\xc5\x03\n\x05Trade\x12\x35\n\x08sayHello\x12\x12.Trade.JSONRequest\x1a\x13.Trade.JSONResponse\"\x00\x12\x32\n\x06getPos\x12\x12.Trade.JSONRequest\x1a\x12.Trade.RspPosition\"\x00\x12\x30\n\tgetOrders\x12\x0f.Trade.ReqQuery\x1a\x10.Trade.RspOrders\"\x00\x12.\n\x08getTrans\x12\x0f.Trade.ReqQuery\x1a\x0f.Trade.RspTrans\"\x00\x12;\n\x0fgetOrder_stream\x12\x12.Trade.JSONRequest\x1a\x10.Trade.RspOrders\"\x00\x30\x01\x12:\n\x0fgetTrans_stream\x12\x12.Trade.JSONRequest\x1a\x0f.Trade.RspTrans\"\x00\x30\x01\x12=\n\x0b\x63\x61ncelOrder\x12\x15.Trade.ReqCancelOrder\x1a\x15.Trade.RspCancelOrder\"\x00\x12\x37\n\tsendOrder\x12\x13.Trade.ReqSentOrder\x1a\x13.Trade.RspSentOrder\"\x00\x62\x06proto3'
)

_RET_TYPE = _descriptor.EnumDescriptor(
  name='RET_TYPE',
  full_name='Trade.RET_TYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERR_NORMAL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERR_SERIOUS', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERR_IGNORE', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1609,
  serialized_end=1681,
)
_sym_db.RegisterEnumDescriptor(_RET_TYPE)

RET_TYPE = enum_type_wrapper.EnumTypeWrapper(_RET_TYPE)
_ORDER_TYPE = _descriptor.EnumDescriptor(
  name='ORDER_TYPE',
  full_name='Trade.ORDER_TYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LIMIT', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MARKET', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1683,
  serialized_end=1718,
)
_sym_db.RegisterEnumDescriptor(_ORDER_TYPE)

ORDER_TYPE = enum_type_wrapper.EnumTypeWrapper(_ORDER_TYPE)
_ORDER_STATUS = _descriptor.EnumDescriptor(
  name='ORDER_STATUS',
  full_name='Trade.ORDER_STATUS',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PENDING', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRADED', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRADED_PART', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CANCEL_REQ', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CANCELED', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REJECTED', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILURE', index=6, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1720,
  serialized_end=1833,
)
_sym_db.RegisterEnumDescriptor(_ORDER_STATUS)

ORDER_STATUS = enum_type_wrapper.EnumTypeWrapper(_ORDER_STATUS)
_TRADE_REPORT_TYPE = _descriptor.EnumDescriptor(
  name='TRADE_REPORT_TYPE',
  full_name='Trade.TRADE_REPORT_TYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NORMAL', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CANCEL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WASTE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1835,
  serialized_end=1889,
)
_sym_db.RegisterEnumDescriptor(_TRADE_REPORT_TYPE)

TRADE_REPORT_TYPE = enum_type_wrapper.EnumTypeWrapper(_TRADE_REPORT_TYPE)
_CANCEL_FLAG = _descriptor.EnumDescriptor(
  name='CANCEL_FLAG',
  full_name='Trade.CANCEL_FLAG',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOT_CANCEL', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IS_CANCEL', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1891,
  serialized_end=1935,
)
_sym_db.RegisterEnumDescriptor(_CANCEL_FLAG)

CANCEL_FLAG = enum_type_wrapper.EnumTypeWrapper(_CANCEL_FLAG)
_DIR = _descriptor.EnumDescriptor(
  name='DIR',
  full_name='Trade.DIR',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BUY', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SELL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BOTH', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NULL', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1937,
  serialized_end=1981,
)
_sym_db.RegisterEnumDescriptor(_DIR)

DIR = enum_type_wrapper.EnumTypeWrapper(_DIR)
SUCCESS = 0
ERR_NORMAL = 1
ERR_SERIOUS = 2
ERR_IGNORE = 3
LIMIT = 0
MARKET = 1
PENDING = 0
TRADED = 1
TRADED_PART = 2
CANCEL_REQ = 3
CANCELED = 4
REJECTED = 5
FAILURE = 6
NORMAL = 0
CANCEL = 1
WASTE = 2
NOT_CANCEL = 0
IS_CANCEL = 1
BUY = 0
SELL = 1
BOTH = 2
NULL = 3



_RSPHEADER = _descriptor.Descriptor(
  name='RspHeader',
  full_name='Trade.RspHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ret_type', full_name='Trade.RspHeader.ret_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sErrInfo', full_name='Trade.RspHeader.sErrInfo', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25,
  serialized_end=89,
)


_JSONREQUEST = _descriptor.Descriptor(
  name='JSONRequest',
  full_name='Trade.JSONRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sReq', full_name='Trade.JSONRequest.sReq', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=118,
)


_JSONRESPONSE = _descriptor.Descriptor(
  name='JSONResponse',
  full_name='Trade.JSONResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rspHeader', full_name='Trade.JSONResponse.rspHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sRsp', full_name='Trade.JSONResponse.sRsp', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=185,
)


_POSITEM = _descriptor.Descriptor(
  name='PosItem',
  full_name='Trade.PosItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sStkCode', full_name='Trade.PosItem.sStkCode', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bsFlat', full_name='Trade.PosItem.bsFlat', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fHoldPrice', full_name='Trade.PosItem.fHoldPrice', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lQty', full_name='Trade.PosItem.lQty', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lFrozenQty', full_name='Trade.PosItem.lFrozenQty', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fAmt', full_name='Trade.PosItem.fAmt', index=5,
      number=6, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sStkName', full_name='Trade.PosItem.sStkName', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=188,
  serialized_end=329,
)


_RSPPOSITION = _descriptor.Descriptor(
  name='RspPosition',
  full_name='Trade.RspPosition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rspHeader', full_name='Trade.RspPosition.rspHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='posItem', full_name='Trade.RspPosition.posItem', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=331,
  serialized_end=414,
)


_ORDERITEM = _descriptor.Descriptor(
  name='OrderItem',
  full_name='Trade.OrderItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sOrderId', full_name='Trade.OrderItem.sOrderId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sStkName', full_name='Trade.OrderItem.sStkName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lOrderTime', full_name='Trade.OrderItem.lOrderTime', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bsFlat', full_name='Trade.OrderItem.bsFlat', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fOrderPrice', full_name='Trade.OrderItem.fOrderPrice', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lOrderQty', full_name='Trade.OrderItem.lOrderQty', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lMatchedQty', full_name='Trade.OrderItem.lMatchedQty', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lWithdrawnQty', full_name='Trade.OrderItem.lWithdrawnQty', index=7,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sStkCode', full_name='Trade.OrderItem.sStkCode', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='iDate', full_name='Trade.OrderItem.iDate', index=9,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='statusInfo', full_name='Trade.OrderItem.statusInfo', index=10,
      number=11, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cancelFlag', full_name='Trade.OrderItem.cancelFlag', index=11,
      number=12, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='iErrCode', full_name='Trade.OrderItem.iErrCode', index=12,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sErrMsg', full_name='Trade.OrderItem.sErrMsg', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fMatchedPrice', full_name='Trade.OrderItem.fMatchedPrice', index=14,
      number=15, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=417,
  serialized_end=768,
)


_RSPORDERS = _descriptor.Descriptor(
  name='RspOrders',
  full_name='Trade.RspOrders',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rspHeader', full_name='Trade.RspOrders.rspHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='orderItem', full_name='Trade.RspOrders.orderItem', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='iTotalSize', full_name='Trade.RspOrders.iTotalSize', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=770,
  serialized_end=875,
)


_REQQUERY = _descriptor.Descriptor(
  name='ReqQuery',
  full_name='Trade.ReqQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='iPageNum', full_name='Trade.ReqQuery.iPageNum', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='iPageSize', full_name='Trade.ReqQuery.iPageSize', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=877,
  serialized_end=924,
)


_TRANSITEM = _descriptor.Descriptor(
  name='TransItem',
  full_name='Trade.TransItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sOrderId', full_name='Trade.TransItem.sOrderId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sMatchedTime', full_name='Trade.TransItem.sMatchedTime', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fMatchedAmt', full_name='Trade.TransItem.fMatchedAmt', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bsFlat', full_name='Trade.TransItem.bsFlat', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fMatchedPrice', full_name='Trade.TransItem.fMatchedPrice', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lMatchedQty', full_name='Trade.TransItem.lMatchedQty', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sMatchedSn', full_name='Trade.TransItem.sMatchedSn', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fTotalCommission', full_name='Trade.TransItem.fTotalCommission', index=7,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tradeReportType', full_name='Trade.TransItem.tradeReportType', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sStkCode', full_name='Trade.TransItem.sStkCode', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sStkName', full_name='Trade.TransItem.sStkName', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=927,
  serialized_end=1204,
)


_RSPTRANS = _descriptor.Descriptor(
  name='RspTrans',
  full_name='Trade.RspTrans',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rspHeader', full_name='Trade.RspTrans.rspHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='transItem', full_name='Trade.RspTrans.transItem', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1206,
  serialized_end=1290,
)


_REQSENTORDER = _descriptor.Descriptor(
  name='ReqSentOrder',
  full_name='Trade.ReqSentOrder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sCode', full_name='Trade.ReqSentOrder.sCode', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bsFlat', full_name='Trade.ReqSentOrder.bsFlat', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='orderType', full_name='Trade.ReqSentOrder.orderType', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='iOrderPrice', full_name='Trade.ReqSentOrder.iOrderPrice', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lOrderQty', full_name='Trade.ReqSentOrder.lOrderQty', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1293,
  serialized_end=1428,
)


_RSPSENTORDER = _descriptor.Descriptor(
  name='RspSentOrder',
  full_name='Trade.RspSentOrder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rspHeader', full_name='Trade.RspSentOrder.rspHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sOrderId', full_name='Trade.RspSentOrder.sOrderId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1430,
  serialized_end=1499,
)


_REQCANCELORDER = _descriptor.Descriptor(
  name='ReqCancelOrder',
  full_name='Trade.ReqCancelOrder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sOrderId', full_name='Trade.ReqCancelOrder.sOrderId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sMarket', full_name='Trade.ReqCancelOrder.sMarket', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1501,
  serialized_end=1552,
)


_RSPCANCELORDER = _descriptor.Descriptor(
  name='RspCancelOrder',
  full_name='Trade.RspCancelOrder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rspHeader', full_name='Trade.RspCancelOrder.rspHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1554,
  serialized_end=1607,
)

_RSPHEADER.fields_by_name['ret_type'].enum_type = _RET_TYPE
_JSONRESPONSE.fields_by_name['rspHeader'].message_type = _RSPHEADER
_POSITEM.fields_by_name['bsFlat'].enum_type = _DIR
_RSPPOSITION.fields_by_name['rspHeader'].message_type = _RSPHEADER
_RSPPOSITION.fields_by_name['posItem'].message_type = _POSITEM
_ORDERITEM.fields_by_name['bsFlat'].enum_type = _DIR
_ORDERITEM.fields_by_name['statusInfo'].enum_type = _ORDER_STATUS
_ORDERITEM.fields_by_name['cancelFlag'].enum_type = _CANCEL_FLAG
_RSPORDERS.fields_by_name['rspHeader'].message_type = _RSPHEADER
_RSPORDERS.fields_by_name['orderItem'].message_type = _ORDERITEM
_TRANSITEM.fields_by_name['bsFlat'].enum_type = _DIR
_TRANSITEM.fields_by_name['tradeReportType'].enum_type = _TRADE_REPORT_TYPE
_RSPTRANS.fields_by_name['rspHeader'].message_type = _RSPHEADER
_RSPTRANS.fields_by_name['transItem'].message_type = _TRANSITEM
_REQSENTORDER.fields_by_name['bsFlat'].enum_type = _DIR
_REQSENTORDER.fields_by_name['orderType'].enum_type = _ORDER_TYPE
_RSPSENTORDER.fields_by_name['rspHeader'].message_type = _RSPHEADER
_RSPCANCELORDER.fields_by_name['rspHeader'].message_type = _RSPHEADER
DESCRIPTOR.message_types_by_name['RspHeader'] = _RSPHEADER
DESCRIPTOR.message_types_by_name['JSONRequest'] = _JSONREQUEST
DESCRIPTOR.message_types_by_name['JSONResponse'] = _JSONRESPONSE
DESCRIPTOR.message_types_by_name['PosItem'] = _POSITEM
DESCRIPTOR.message_types_by_name['RspPosition'] = _RSPPOSITION
DESCRIPTOR.message_types_by_name['OrderItem'] = _ORDERITEM
DESCRIPTOR.message_types_by_name['RspOrders'] = _RSPORDERS
DESCRIPTOR.message_types_by_name['ReqQuery'] = _REQQUERY
DESCRIPTOR.message_types_by_name['TransItem'] = _TRANSITEM
DESCRIPTOR.message_types_by_name['RspTrans'] = _RSPTRANS
DESCRIPTOR.message_types_by_name['ReqSentOrder'] = _REQSENTORDER
DESCRIPTOR.message_types_by_name['RspSentOrder'] = _RSPSENTORDER
DESCRIPTOR.message_types_by_name['ReqCancelOrder'] = _REQCANCELORDER
DESCRIPTOR.message_types_by_name['RspCancelOrder'] = _RSPCANCELORDER
DESCRIPTOR.enum_types_by_name['RET_TYPE'] = _RET_TYPE
DESCRIPTOR.enum_types_by_name['ORDER_TYPE'] = _ORDER_TYPE
DESCRIPTOR.enum_types_by_name['ORDER_STATUS'] = _ORDER_STATUS
DESCRIPTOR.enum_types_by_name['TRADE_REPORT_TYPE'] = _TRADE_REPORT_TYPE
DESCRIPTOR.enum_types_by_name['CANCEL_FLAG'] = _CANCEL_FLAG
DESCRIPTOR.enum_types_by_name['DIR'] = _DIR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RspHeader = _reflection.GeneratedProtocolMessageType('RspHeader', (_message.Message,), {
  'DESCRIPTOR' : _RSPHEADER,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.RspHeader)
  })
_sym_db.RegisterMessage(RspHeader)

JSONRequest = _reflection.GeneratedProtocolMessageType('JSONRequest', (_message.Message,), {
  'DESCRIPTOR' : _JSONREQUEST,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.JSONRequest)
  })
_sym_db.RegisterMessage(JSONRequest)

JSONResponse = _reflection.GeneratedProtocolMessageType('JSONResponse', (_message.Message,), {
  'DESCRIPTOR' : _JSONRESPONSE,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.JSONResponse)
  })
_sym_db.RegisterMessage(JSONResponse)

PosItem = _reflection.GeneratedProtocolMessageType('PosItem', (_message.Message,), {
  'DESCRIPTOR' : _POSITEM,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.PosItem)
  })
_sym_db.RegisterMessage(PosItem)

RspPosition = _reflection.GeneratedProtocolMessageType('RspPosition', (_message.Message,), {
  'DESCRIPTOR' : _RSPPOSITION,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.RspPosition)
  })
_sym_db.RegisterMessage(RspPosition)

OrderItem = _reflection.GeneratedProtocolMessageType('OrderItem', (_message.Message,), {
  'DESCRIPTOR' : _ORDERITEM,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.OrderItem)
  })
_sym_db.RegisterMessage(OrderItem)

RspOrders = _reflection.GeneratedProtocolMessageType('RspOrders', (_message.Message,), {
  'DESCRIPTOR' : _RSPORDERS,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.RspOrders)
  })
_sym_db.RegisterMessage(RspOrders)

ReqQuery = _reflection.GeneratedProtocolMessageType('ReqQuery', (_message.Message,), {
  'DESCRIPTOR' : _REQQUERY,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.ReqQuery)
  })
_sym_db.RegisterMessage(ReqQuery)

TransItem = _reflection.GeneratedProtocolMessageType('TransItem', (_message.Message,), {
  'DESCRIPTOR' : _TRANSITEM,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.TransItem)
  })
_sym_db.RegisterMessage(TransItem)

RspTrans = _reflection.GeneratedProtocolMessageType('RspTrans', (_message.Message,), {
  'DESCRIPTOR' : _RSPTRANS,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.RspTrans)
  })
_sym_db.RegisterMessage(RspTrans)

ReqSentOrder = _reflection.GeneratedProtocolMessageType('ReqSentOrder', (_message.Message,), {
  'DESCRIPTOR' : _REQSENTORDER,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.ReqSentOrder)
  })
_sym_db.RegisterMessage(ReqSentOrder)

RspSentOrder = _reflection.GeneratedProtocolMessageType('RspSentOrder', (_message.Message,), {
  'DESCRIPTOR' : _RSPSENTORDER,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.RspSentOrder)
  })
_sym_db.RegisterMessage(RspSentOrder)

ReqCancelOrder = _reflection.GeneratedProtocolMessageType('ReqCancelOrder', (_message.Message,), {
  'DESCRIPTOR' : _REQCANCELORDER,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.ReqCancelOrder)
  })
_sym_db.RegisterMessage(ReqCancelOrder)

RspCancelOrder = _reflection.GeneratedProtocolMessageType('RspCancelOrder', (_message.Message,), {
  'DESCRIPTOR' : _RSPCANCELORDER,
  '__module__' : 'tradeApi_pb2'
  # @@protoc_insertion_point(class_scope:Trade.RspCancelOrder)
  })
_sym_db.RegisterMessage(RspCancelOrder)



_TRADE = _descriptor.ServiceDescriptor(
  name='Trade',
  full_name='Trade.Trade',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1984,
  serialized_end=2437,
  methods=[
  _descriptor.MethodDescriptor(
    name='sayHello',
    full_name='Trade.Trade.sayHello',
    index=0,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_JSONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getPos',
    full_name='Trade.Trade.getPos',
    index=1,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_RSPPOSITION,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getOrders',
    full_name='Trade.Trade.getOrders',
    index=2,
    containing_service=None,
    input_type=_REQQUERY,
    output_type=_RSPORDERS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getTrans',
    full_name='Trade.Trade.getTrans',
    index=3,
    containing_service=None,
    input_type=_REQQUERY,
    output_type=_RSPTRANS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getOrder_stream',
    full_name='Trade.Trade.getOrder_stream',
    index=4,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_RSPORDERS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getTrans_stream',
    full_name='Trade.Trade.getTrans_stream',
    index=5,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_RSPTRANS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='cancelOrder',
    full_name='Trade.Trade.cancelOrder',
    index=6,
    containing_service=None,
    input_type=_REQCANCELORDER,
    output_type=_RSPCANCELORDER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='sendOrder',
    full_name='Trade.Trade.sendOrder',
    index=7,
    containing_service=None,
    input_type=_REQSENTORDER,
    output_type=_RSPSENTORDER,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TRADE)

DESCRIPTOR.services_by_name['Trade'] = _TRADE

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from rpc.Tradex import tradex_pb2 as tradex__pb2


class TradeXStub(object):
  """The greeting service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.SayHello = channel.unary_unary(
        '/TradeX.TradeX/SayHello',
        request_serializer=tradex__pb2.JSONRequest.SerializeToString,
        response_deserializer=tradex__pb2.JSONResponse.FromString,
        )
    self.GetPos = channel.unary_unary(
        '/TradeX.TradeX/GetPos',
        request_serializer=tradex__pb2.JSONRequest.SerializeToString,
        response_deserializer=tradex__pb2.JSONResponse.FromString,
        )
    self.GetOrders = channel.unary_unary(
        '/TradeX.TradeX/GetOrders',
        request_serializer=tradex__pb2.JSONRequest.SerializeToString,
        response_deserializer=tradex__pb2.JSONResponse.FromString,
        )
    self.SendOrder = channel.unary_unary(
        '/TradeX.TradeX/SendOrder',
        request_serializer=tradex__pb2.JSONRequest.SerializeToString,
        response_deserializer=tradex__pb2.JSONResponse.FromString,
        )


class TradeXServicer(object):
  """The greeting service definition.
  """

  def SayHello(self, request, context):
    """Sends a greeting
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetPos(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetOrders(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SendOrder(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_TradeXServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'SayHello': grpc.unary_unary_rpc_method_handler(
          servicer.SayHello,
          request_deserializer=tradex__pb2.JSONRequest.FromString,
          response_serializer=tradex__pb2.JSONResponse.SerializeToString,
      ),
      'GetPos': grpc.unary_unary_rpc_method_handler(
          servicer.GetPos,
          request_deserializer=tradex__pb2.JSONRequest.FromString,
          response_serializer=tradex__pb2.JSONResponse.SerializeToString,
      ),
      'GetOrders': grpc.unary_unary_rpc_method_handler(
          servicer.GetOrders,
          request_deserializer=tradex__pb2.JSONRequest.FromString,
          response_serializer=tradex__pb2.JSONResponse.SerializeToString,
      ),
      'SendOrder': grpc.unary_unary_rpc_method_handler(
          servicer.SendOrder,
          request_deserializer=tradex__pb2.JSONRequest.FromString,
          response_serializer=tradex__pb2.JSONResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'TradeX.TradeX', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# -*- coding: utf-8 -*-
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC helloworld.Greeter client."""

from __future__ import print_function
import logging

import grpc

from rpc.Tradex import  tradex_pb2
from rpc.Tradex import  tradex_pb2_grpc
import json
TOKEN = "WQiOiJLNzFVOERCUE5FIiw"
HOST = "106.75.213.173:50051"

def run():
    # NOTE(gRPC Python Team): .close() is possible on a channel and should be
    # used in circumstances in which the with statement does not fit the needs
    # of the code.
    with grpc.insecure_channel(HOST) as channel:
        stub = tradex_pb2_grpc.TradeXStub(channel)
        rqt_string = json.dumps({"token": TOKEN,
                                 "name": 'hanc',})
        # SayHello
        print("test rpcfunc SayHello")
        response = stub.SayHello(tradex_pb2.JSONRequest(rqt_string=rqt_string))
        print(response.rsp_string)

        print("test rpcfunc GetPos")
        rqt_string = json.dumps({"token": TOKEN,
                                 })
        response = stub.GetPos(tradex_pb2.JSONRequest(rqt_string=rqt_string))
        print(response.rsp_string)

    print("test rpcfunc GetOrders")
    orders = get_orders()
    print(orders)


    nCategory= 0 # 0买入, 1卖出
    nOrderType = 0 # 0限价委托； 上海限价委托 / 深圳限价委托
    stock_code="000001"
    issue_price=float(10.27)
    apply_cnt =1000000

    print("test SendOrder GetOrders")
    res = send_order(nCategory, nOrderType, stock_code, issue_price, apply_cnt)
    print(res)



def get_orders():
    with grpc.insecure_channel(HOST) as channel:
        stub = tradex_pb2_grpc.TradeXStub(channel)
        rqt_string = json.dumps({"token": TOKEN,
                                 "name": 'hanc',})
        orders = stub.GetOrders(tradex_pb2.JSONRequest(rqt_string=rqt_string))
        d = json.loads(orders.rsp_string)
        if d["status"] == -1:
            print(d["err_text"])
            return
        return d["res"]

 # nCategory= 0 # 0买入, 1卖出
 #nOrderType = 0 # 0限价委托； 上海限价委托 / 深圳限价委托
 #stock_code="000001"
 #issue_price=float(10)
 #apply_cnt =100

def send_order( nCategory, nOrderType, stock_code, issue_price, apply_cnt):
    with grpc.insecure_channel(HOST) as channel:
        stub = tradex_pb2_grpc.TradeXStub(channel)
        rqt_string = json.dumps({"token": TOKEN,
                                 "order_param": {"stock_code": stock_code,
                                                 "nCategory": nCategory,
                                                 "nOrderType": nOrderType,
                                                 "issue_price": issue_price,
                                                 "apply_cnt": apply_cnt,
                                                 }}

                                )
        res = stub.SendOrder(tradex_pb2.JSONRequest(rqt_string=rqt_string))

        d = json.loads(res.rsp_string)
        if d["status"] == -1:
            print(d["err_text"])
            return d["err_text"]
        return d["res"]


if __name__ == '__main__':
    logging.basicConfig()
    run()


from typing import List
from dataclasses import dataclass
from common.define import ( TRADE_TYPE,
                            ORDER_TYPE,
                            ORDER_STATUS,
                            MARKET,
                            DIR)

@dataclass
class BaseData:
    """
    Any data object needs a gateway_name as source or
    destination and should inherit base data.
    """
    gateway_name: MARKET

@dataclass
class stBarData(BaseData):
    """
    Candlestick bar data of a certain trading period.
    """
    code: str
    date: int
    time: int
    volume: float = 0
    open_price: float = 0
    high_price: float = 0
    low_price: float = 0
    close_price: float = 0

@dataclass
class stSendOrder(BaseData):
    """报单"""
    code: str
    order_type: ORDER_TYPE
    trade_type: TRADE_TYPE
    dir: DIR
    price: float
    number: int
    date: int
    time: int
    name: str
    local_no: str
    order_no: str = ''


@dataclass
class stCancelOrder(BaseData):
    """撤单"""
    order_no: str
    market: int
    status: ORDER_STATUS


@dataclass
class stPositionsRequest(BaseData):
    """持仓请求"""
    pass

@dataclass
class stTransRequest(BaseData):
    """成交请求"""
    pass

@dataclass
class stOrdersRequest(BaseData):
    """报单请求"""
    pass


@dataclass
class stOrderStatus(BaseData):
    """报单状态"""
    order_no: str       # 委托编号
    status: ORDER_STATUS


@dataclass
class stPos:
    """持仓信息"""
    code: str
    price: float
    number: int
    dir: DIR
    margin: float           # 市值
    unlimited_sell: int =0  # 可卖数量(昨仓数量
    name: str = ''          # 合约名字
    day_counter: int =0     # 持仓时间
    datetime_last: int =0   # 加仓/减仓时间
    datetime_open: int =0   # 开仓时间

@dataclass
class stPositions(BaseData):
    """持仓集合"""
    datas: List[stPos]

@dataclass
class stOrder:
    """报单信息"""
    code: str
    name: str
    dir: DIR
    trade_type: TRADE_TYPE
    date:int
    order_time: int     # 委托时间
    price: float        # 委托价格
    price_traded: float # 成交价格
    number: int         # 委托数量
    number_traded: int  # 成交数量
    number_cancel: int  # 撤单数量
    order_no: str       # 委托编号
    local_no: str       # 本地编号
    status: ORDER_STATUS# 委托状态

@dataclass
class stOrders(BaseData):
    """报单集合"""
    datas: List[stOrder]


@dataclass
class stTran:
    """成交记录"""
    market:MARKET
    code:str
    date:int
    tran_time:int
    dir:DIR
    trade_type: TRADE_TYPE
    price:float
    commission:float
    number_traded:int
    order_no:str           # 委托编号
    tran_no:str            # 成交编号
    turnover:float         # 发生金额
    op_name:str = ''       # 业务名称
    name:str = ''
    trade_code:str = ''     # 交易代码


@dataclass
class stTrans(BaseData):
    """成交集合"""
    datas: List[stTran]


@dataclass
class stProfit:
    """利润记录"""
    code:str
    date:int
    time: int
    dir: DIR
    price: float
    number: int
    tran_no: str   # 成交编号
    profit: float
    commission: float
    name:str = ''



# -*- coding: utf-8 -*-
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC helloworld.Greeter server."""

from concurrent import futures
import time
import logging
import json
import grpc

import tradex_pb2
import tradex_pb2_grpc

from stock_tools.TradeX import _TradeX as TradeX


_ONE_DAY_IN_SECONDS = 1
# ACCOUNT = "35212133"
# PW = "159159"
ACCOUNT = "8660001779"
PW = "188188"
TOKEN = "WQiOiJLNzFVOERCUE5FIiw"

# 用于验证客户端的请求是否合法
def isAuth(token):

    if token == TOKEN:
        return True

class TradeXServicer(tradex_pb2_grpc.TradeXServicer):

    def SayHello(self, request, context):
        rqt_param = json.loads(request.rqt_string)
        if not isAuth(rqt_param["token"]):
            return json.dumps({"status": -1, "err_text": "非法访问"})
        return tradex_pb2.HelloReply(message='Hello, %s!' % rqt_param["name"])


    def GetPos(self, request, context):
        rqt_param = json.loads(request.rqt_string)
        if not isAuth(rqt_param["token"]):
            return json.dumps({"status": -1, "err_text": "非法访问"})
        tdx = TradeX._TradeX(ACCOUNT)
        tdx.open_Tdx()
        client_Id, e = tdx.logon(PW)
        json_response = tradex_pb2.JSONResponse()
        if e :
            json_response.rsp_string = json.dumps({"status":-1,"err_text":e})
            return json_response
        pos =  tdx.get_stock_positions()
        json_response.rsp_string=json.dumps({"status":0,"res":pos})
        return json_response

    def GetOrders(self, request, context):
        rqt_param = json.loads(request.rqt_string)
        if not isAuth(rqt_param["token"]):
            return json.dumps({"status": -1, "err_text": "非法访问"})
        tdx = TradeX._TradeX(ACCOUNT)
        tdx.open_Tdx()
        client_Id, e = tdx.logon(PW)
        json_response = tradex_pb2.JSONResponse()
        if e :
            json_response.rsp_string = json.dumps({"status":-1,"err_text":e})
            return json_response
        orders = tdx.get_Orders()
        json_response.rsp_string=json.dumps({"status":0,"res":orders})
        return json_response

    def SendOrder(self, request, context):
        rqt_param = json.loads(request.rqt_string)
        if not isAuth(rqt_param["token"]):
            return json.dumps({"status": -1, "err_text": "非法访问"})
        tdx = TradeX._TradeX(ACCOUNT)
        tdx.open_Tdx()
        client_Id, e = tdx.logon(PW)
        json_response = tradex_pb2.JSONResponse()
        if e :
            json_response.rsp_string = json.dumps({"status":-1,"err_text":e})
            return json_response

        order_param  = rqt_param["order_param"]
        print(type(order_param))
        r, e = tdx.SendOrder(order_param["nOrderType"], order_param["nCategory"], order_param["stock_code"], order_param["issue_price"], order_param["apply_cnt"])
        if e:
            json_response.rsp_string = json.dumps({"status": -1, "err_text": e})
            return json_response
        json_response.rsp_string=json.dumps({"status":0,"res":r})
        return json_response

def serve():
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    tradex_pb2_grpc.add_TradeXServicer_to_server(TradeXServicer(), server)
    server.add_insecure_port('[::]:50051')
    server.start()
    try:
        while True:
            time.sleep(_ONE_DAY_IN_SECONDS)
    except KeyboardInterrupt:
        server.stop(0)


if __name__ == '__main__':
    logging.basicConfig()
    serve()



import setuptools

# 第三方依赖包及版本
requires = [
    'chardet>=3.0.4',
    'dataclasses>=0.7',
    'Twisted>=20.3.0',
    'urllib3>=1.22',
    'PyMySQL>=0.9.2',
    'psutil>=5.4.5',
    'requests>=2.18.4',
    'xlrd>=1.1.0',
    'colorama>=0.3.9',
    'pandas>=0.23.0',
    'six>=1.11.0',
    'pywin32>=223',
    'tushare>=1.2.60',
    'numpy>=1.14.3',
    'stompest.async>=2.3.0',
    # 'TA_Lib>=0.4.18',
    'scipy>=1.1.0',
    'grpcio>=1.29.0',
    'backports.statistics>=0.1.0',
    'beautifulsoup4>=4.9.3',
    'cached_property>=1.5.2',
    'pymongo',
    'manga>=0.1.13',
    'protobuf>=3.14.0',
    'pycryptodome>=3.9.9',
    'python_dateutil>=2.8.1',
    'redis>=3.5.3',
    'rqalpha>=4.2.4',
    'rqdatac>=2.9.15',
    'stompest>=2.3.0'
]

with open("README.md", "r") as fh:
    long_description = fh.read()
    
setuptools.setup(name='tmqc',
    version='0.1.2',
    description='tian ma quant cloud',
    author='tmqc',
    author_email='tmqc@gmail.com',
    url="https://github.com/tmqc/tmqc",
    license='LGPT',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    python_requires='==3.6.5',
    install_requires = requires,
    include_package_data=True,
    zip_safe=False)


# -*- coding: utf-8 -*-
import re
import datetime
import math
import os
import sys
import traceback
from common import pyetc
from common import db
from common import define
from datetime import datetime as dtime
from common.define import (MARKET)
import os
# 交易日期类型
TRADE_DATE_TYPE = define.enum(month=0, quarter=1, halfyear=2)

def atoi(s):
    try:
        return int(s)
    except Exception as e:
        #print(e)
        return 0

def format_traceback():
    lines = []
    lines.extend(traceback.format_stack())
    lines.extend(traceback.format_tb(sys.exc_info()[2]))
    lines.extend(traceback.format_exception_only(sys.exc_info()[0], sys.exc_info()[1]))

    join_line = ["Traceback (most recent call last):\n"]
    for strg in lines:
        if not isinstance(strg, str):
            join_line.append(strg.decode("utf8"))
        else:
            join_line.append(strg)

    except_str = ''.join(join_line)

    # Removing the last \n
    except_str = except_str[:-1]
    return except_str

def gen_path(STOCK_CODE_PATH):
    if getattr(sys, 'frozen', False):
        pathname = STOCK_CODE_PATH
    else:
        pathname = os.path.join(os.path.dirname(os.path.dirname(__file__)), STOCK_CODE_PATH)
    return pathname

def format_datetime(datetime:str):
    length = len(str(datetime))
    if length == 17:
        return "%s-%s-%s %s:%s:%s.%s" % (
             datetime[:4], datetime[4:6], datetime[6:8], datetime[8:10], datetime[10:12], datetime[12:14], datetime[14:]
        )
    elif length == 9:
        return "%s:%s:%s.%s"% (datetime[0:2],datetime[2:4],datetime[4:6],datetime[6:])
    else:
        return

# 方便兼容打包exe
def get_path_dirname():
    if getattr(sys, 'frozen', False):
        application_path = os.path.dirname(sys.executable)
    else:
        # 取当前脚本的上级目录
        application_path = os.path.dirname(os.path.dirname(__file__))
    return application_path


def get_db_info():
    path = "conf" + os.sep + "db.conf"
    work_dir = get_path_dirname()
    full_path = work_dir + os.sep + path

    setting = pyetc.load(full_path)
    return setting.db_info


def get_strategy_setting():
    path = "conf" + os.sep + "Strategy_conf.py"
    work_dir = get_path_dirname()
    full_path = work_dir + os.sep + path

    setting = pyetc.load(full_path)
    return setting

def get_strategy_conf():
    path = ["conf","strategy_stk","stk.xlsx"]
    path = os.sep.join(path)
    work_dir = get_path_dirname()
    full_path = work_dir + os.sep + path

    import xlrd
    # 1、打开文件
    with xlrd.open_workbook(full_path) as f:
        sheet = f.sheet_by_index(0)
        codes =sheet.col_values(0)

    return codes


def create_database(info=None, table=''):
    if not info:
        info = get_db_info()
    print("connect %s"%info['host'])
    return db.DB(host=info['host'], user=info['user'], passwd=info['password'], db=info['db'], table=table, charset=info['charset'],
                 port=info['port'])


def make_date(date_str):
    if date_str == '--': return 0
    date_arr = date_str.split('-')

    if len(date_arr) == 3:
        return int(date_arr[0]) * 10000 + int(date_arr[1]) * 100 + int(date_arr[2])
    return None


def get_number(str):
    if str == '--':    return 0
    m = re.match('(-{0,1}\d+\.{0,1}\d*).*', str)
    if m:
        return float(m.groups()[0])
    else:
        return 0


def get_bignumber(str):
    if str == '--':    return 0
    m = re.match('(-{0,1}\d+[,\d+]*\.{0,1}\d*).*', str)
    if m:
        return float(m.groups()[0].replace(',', ''))
    else:
        return 0


def parse_tds(trs):
    tds = trs.findAll("td")
    return [tdc.text for tdc in tds]


# 找不同
def find_diff(base, iters):
    return [s for s in iters if s not in base]


# 向前获取最近的财报日期
# date_type： 0：月报；1：季报；2：半年报
def get_pre_date_nearby(datetime, date_type=TRADE_DATE_TYPE.quarter):
    date_range = [131, 230, 331, 430, 531, 630, 731, 831, 930, 1031, 1130, 1231]
    if TRADE_DATE_TYPE.quarter == date_type:
        date_range = [331, 630, 930, 1231]
    elif TRADE_DATE_TYPE.halfyear == date_type:
        date_range = [630, 1231]

    mmdd = datetime % 10000
    yyyy = datetime // 10000
    calc_datetime = 0
    for d in date_range[::-1]:
        # if mmdd > d+100: # 因为季报推迟1个月才会公布。
        if mmdd > d:
            calc_datetime = d
            break
    if calc_datetime == 0:
        # calc_datetime =  date_range[-1]  if mmdd/100>2 else date_range[-2] # 因为季报推迟1个月才会公布。
        calc_datetime = date_range[-1:][0]
        yyyy -= 1
    return yyyy * 10000 + calc_datetime


def get_last_year_finance_report_date(datetime):
    dates = []
    date_range = [1231, 930, 630, 331]
    pre_date = get_pre_date_nearby(datetime)
    dates.append(pre_date)
    md = pre_md = pre_date % 10000
    year = pre_y = pre_date / 10000
    beg_index = date_range.index(pre_md)
    for index in range(len(date_range)):
        beg_index += 1
        if beg_index == len(date_range):
            beg_index = 0
            year -= 1
        md = date_range[beg_index]
        if md == pre_md: break
        dates.append(year * 10000 + md)
    return dates


# 向后获取最近的日期
def get_next_date_nearby(datetime, date_type=TRADE_DATE_TYPE.quarter):
    date_range = [131, 230, 331, 430, 531, 630, 731, 831, 930, 1031, 1130, 1231]
    if TRADE_DATE_TYPE.quarter == date_type:
        date_range = [331, 630, 930, 1231]
    elif TRADE_DATE_TYPE.halfyear == date_type:
        date_range = [630, 1231]

    mmdd = datetime % 10000
    yyyy = datetime / 10000
    calc_datetime = 0
    for d in date_range:
        if mmdd < d:
            calc_datetime = d
            break

    if calc_datetime == 0:
        calc_datetime = date_range[0]
        yyyy += 1
    return yyyy * 10000 + calc_datetime


# def get_offset_datetime(dt, offset):  # 得到以dt为基准日往前(后)偏差的日期  offset负为往前
#     base_day = datetime.datetime(dt // 10000, dt // 100 % 100, dt % 100)
#     return int((base_day + datetime.timedelta(offset)).strftime('%Y%m%d'))

def get_offset_datetime(dt, offset):
    """得到以dt为基准日往前(后)偏差的日期  offset负为往前.根据时间传参判断偏移计算单位"""
    from dateutil.parser import parse
    dt_star = parse(str(dt))
    type_ = get_datetime_type(dt)
    if type_ == "minutes":
        date = dt_star+datetime.timedelta(minutes=offset)
        return int(date.strftime('%Y%m%d%H%M'))
    elif type_ == "day":
        date = dt_star + datetime.timedelta(days=offset)
        return  int(date.strftime('%Y%m%d'))
    else:
        print("获取时间类型异常")

def get_days_interval(a_dt, b_dt):  # 获取两个交易日偏差多少天
    a_day = datetime.datetime(a_dt // 10000, a_dt // 100 % 100, a_dt % 100)
    b_day = datetime.datetime(b_dt // 10000, b_dt // 100 % 100, b_dt % 100)
    timedelta = a_day - b_day
    return abs(timedelta.days)


def get_week_day(a_dt):  # 星期几
    a_day = datetime.datetime(a_dt / 10000, a_dt / 100 % 100, a_dt % 100)
    return a_day.weekday() + 1


def get_avarage(datas):  # 获取平均数
    if not datas or len(datas) == 0: return 0
    total = 0
    for data in datas:
        total += data
    return total / float(len(datas))


def get_variance(datas):
    if not datas or len(datas) == 0: return 0
    avarage = get_avarage(datas)
    total = 0
    for data in datas:
        total += (data - avarage) * (data - avarage)
    return total / float(len(datas))


def get_standard_deviation(datas):
    if not datas or len(datas) == 0: return 0
    variance = get_variance(datas)
    return math.sqrt(variance)


def is_trade_local_time(now_time):
    hhmm = now_time % 10000
    weekday = datetime.date.today().weekday()
    if weekday == 5 or weekday == 6: return False
    return True


def is_trading_time(hhmm :int, market:MARKET):    
    if market == MARKET.stock or market == MARKET.option:
        if hhmm >= 928 and hhmm <= 1132: return True
        if hhmm >= 1258 and hhmm <= 1502: return True
    elif market == MARKET.future:
        if hhmm >= 928 and hhmm <= 1132: return True
        if hhmm >= 1258 and hhmm <= 1502: return True
        if hhmm >= 2058 and hhmm <= 2400: return True
        if hhmm >= 0 and hhmm <= 300: return True
    return False


def gen_datetime_range(beg, end, delta):
    datetime_list=[]
    interval = get_days_interval(beg, end)

    if interval <= delta:
        datetime_list.append((beg, end))
        return datetime_list

    list_num = interval//delta+1
    for n in range(list_num):
        _beg = get_offset_datetime(beg, delta*n)
        _end = get_offset_datetime(beg, delta*(n+1)-1)
        _end = _end if _end < end else end
        datetime_list.append((_beg,_end))

    return datetime_list

def get_datetime_type(datetime):
    import math
    len = int(math.log10(datetime))+1
    _type={ 8:"day",
            12:"minutes",
            9:"tick"}
    return _type[len]

def strptime(datetime):
    _t = get_datetime_type(datetime)
    if _t == "day": datetime *=10000
    if isinstance(datetime, int):
        _dt = str(datetime)
    else:
        _dt = datetime
    return dtime.strptime(_dt, '%Y%m%d%H%M')


def calc_time_diff(beg, end=None):
    b = datetime.datetime.strptime(str(beg), '%Y%m%d%H%M%S%f')
    if end:
        e = datetime.datetime.strptime(str(end), '%Y%m%d%H%M%S%f')
    else:
        e = datetime.datetime.now()
    seconds_diff = (e - b).total_seconds()
    return seconds_diff


def gen_new_time(beg,**kwargs):
    beg = str(beg)
    beg = datetime.datetime.strptime(beg, "%H%M%S%f")
    # print(times1)
    new_time = int((beg + datetime.timedelta(**kwargs)).strftime('%H%M%S%f')[:-3])
    return new_time


def varname(p):
    """获取变量名字的字符串"""
    import inspect, re
    for line in inspect.getframeinfo(inspect.currentframe().f_back)[3]:
        m = re.search(r'\bvarname\s*\(\s*([A-Za-z_][A-Za-z0-9_]*\.)*([A-Za-z_][A-Za-z0-9_]*)\s*\)', line)
    if m: return m.group(2)


def reduce_mem_usage(props):
    import numpy as np

    """收缩DataFrame 内存，类型自动向下转换"""
    # 计算当前内存
    start_mem_usg = props.memory_usage().sum() / 1024 ** 2
    print("Memory usage of the dataframe is :", start_mem_usg, "MB")

    # 哪些列包含空值，空值用-999填充。why：因为np.nan当做float处理
    NAlist = []
    for col in props.columns:
        # 这里只过滤了objectd格式，如果你的代码中还包含其他类型，请一并过滤
        if (props[col].dtypes != object):

            print("**************************")
            print("columns: ", col)
            print("dtype before", props[col].dtype)

            # 判断是否是int类型
            isInt = False
            mmax = props[col].max()
            mmin = props[col].min()

            # Integer does not support NA, therefore Na needs to be filled
            if not np.isfinite(props[col]).all():
                NAlist.append(col)
                props[col].fillna(-999, inplace=True)  # 用-999填充

            # test if column can be converted to an integer
            asint = props[col].fillna(0).astype(np.int64)
            result = np.fabs(props[col] - asint)
            result = result.sum()
            if result < 0.01:  # 绝对误差和小于0.01认为可以转换的，要根据task修改
                isInt = True

            # make interger / unsigned Integer datatypes
            if isInt:
                if mmin >= 0:  # 最小值大于0，转换成无符号整型
                    if mmax <= 255:
                        props[col] = props[col].astype(np.uint8)
                    elif mmax <= 65535:
                        props[col] = props[col].astype(np.uint16)
                    elif mmax <= 4294967295:
                        props[col] = props[col].astype(np.uint32)
                    else:
                        props[col] = props[col].astype(np.uint64)
                else:  # 转换成有符号整型
                    if mmin > np.iinfo(np.int8).min and mmax < np.iinfo(np.int8).max:
                        props[col] = props[col].astype(np.int8)
                    elif mmin > np.iinfo(np.int16).min and mmax < np.iinfo(np.int16).max:
                        props[col] = props[col].astype(np.int16)
                    elif mmin > np.iinfo(np.int32).min and mmax < np.iinfo(np.int32).max:
                        props[col] = props[col].astype(np.int32)
                    elif mmin > np.iinfo(np.int64).min and mmax < np.iinfo(np.int64).max:
                        props[col] = props[col].astype(np.int64)
            else:  # 注意：这里对于float都转换成float16，需要根据你的情况自己更改
                props[col] = props[col].astype(np.float16)

            print("dtype after", props[col].dtype)
            print("********************************")
    print("___MEMORY USAGE AFTER COMPLETION:___")
    mem_usg = props.memory_usage().sum() / 1024 ** 2
    print("Memory usage is: ", mem_usg, " MB")
    print("This is ", 100 * mem_usg / start_mem_usg, "% of the initial size")
    return props, NAlist


# 获得第几季度
def get_quarter(datetime):
    quarter_range = [331,630,930,1231]
    mm_dd = datetime%10000
    for idx,quarter_ in enumerate(quarter_range):
        if mm_dd<=quarter_:
            return  idx+1
    print("get_quarter err")



import sys
import traceback
import os

def atoi(s):
	try:
		return int(s)
	except Exception as e:
		#print(e)
		return 0

def format_traceback():
	lines = []
	lines.extend(traceback.format_stack())
	lines.extend(traceback.format_tb(sys.exc_info()[2]))
	lines.extend(traceback.format_exception_only(sys.exc_info()[0], sys.exc_info()[1]))

	join_line = ["Traceback (most recent call last):\n"]
	for strg in lines:
		if not isinstance(strg, str):
			join_line.append(strg.decode("utf8"))
		else:
			join_line.append(strg)

	except_str = ''.join(join_line)

	# Removing the last \n
	except_str = except_str[:-1]
	return except_str

def gen_path(STOCK_CODE_PATH):
	if getattr(sys, 'frozen', False):
		pathname = STOCK_CODE_PATH
	else:
		pathname = os.path.join(os.path.dirname(os.path.dirname(__file__)), STOCK_CODE_PATH)
	return pathname

def format_datetime(datetime:str):
	length = len(str(datetime))
	if length == 17:
		return "%s-%s-%s %s:%s:%s.%s" % (
             datetime[:4], datetime[4:6], datetime[6:8], datetime[8:10], datetime[10:12], datetime[12:14], datetime[14:]
		)
	elif length == 9:
		return "%s:%s:%s.%s"% (datetime[0:2],datetime[2:4],datetime[4:6],datetime[6:])
	else:
		return


# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: indexData.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='indexData.proto',
  package='indexData',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0findexData.proto\x12\tindexData\"D\n\tRspHeader\x12%\n\x08ret_type\x18\x01 \x01(\x0e\x32\x13.indexData.RET_TYPE\x12\x10\n\x08sErrInfo\x18\x02 \x01(\t\"\x1b\n\x0bJSONRequest\x12\x0c\n\x04sReq\x18\x01 \x01(\t\"E\n\x0cJSONResponse\x12\'\n\trspHeader\x18\x01 \x01(\x0b\x32\x14.indexData.RspHeader\x12\x0c\n\x04sRsp\x18\x02 \x01(\t\"B\n\x07Symbols\x12\'\n\trspHeader\x18\x01 \x01(\x0b\x32\x14.indexData.RspHeader\x12\x0e\n\x06symbol\x18\x02 \x03(\t*H\n\x08RET_TYPE\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0e\n\nERR_NORMAL\x10\x01\x12\x0f\n\x0b\x45RR_SERIOUS\x10\x02\x12\x0e\n\nERR_IGNORE\x10\x03\x32\x86\x01\n\tindexData\x12=\n\x08sayHello\x12\x16.indexData.JSONRequest\x1a\x17.indexData.JSONResponse\"\x00\x12:\n\ngetSymbols\x12\x16.indexData.JSONRequest\x1a\x12.indexData.Symbols\"\x00\x62\x06proto3')
)

_RET_TYPE = _descriptor.EnumDescriptor(
  name='RET_TYPE',
  full_name='indexData.RET_TYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERR_NORMAL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERR_SERIOUS', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERR_IGNORE', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=268,
  serialized_end=340,
)
_sym_db.RegisterEnumDescriptor(_RET_TYPE)

RET_TYPE = enum_type_wrapper.EnumTypeWrapper(_RET_TYPE)
SUCCESS = 0
ERR_NORMAL = 1
ERR_SERIOUS = 2
ERR_IGNORE = 3



_RSPHEADER = _descriptor.Descriptor(
  name='RspHeader',
  full_name='indexData.RspHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ret_type', full_name='indexData.RspHeader.ret_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sErrInfo', full_name='indexData.RspHeader.sErrInfo', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30,
  serialized_end=98,
)


_JSONREQUEST = _descriptor.Descriptor(
  name='JSONRequest',
  full_name='indexData.JSONRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sReq', full_name='indexData.JSONRequest.sReq', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=100,
  serialized_end=127,
)


_JSONRESPONSE = _descriptor.Descriptor(
  name='JSONResponse',
  full_name='indexData.JSONResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rspHeader', full_name='indexData.JSONResponse.rspHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sRsp', full_name='indexData.JSONResponse.sRsp', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=129,
  serialized_end=198,
)


_SYMBOLS = _descriptor.Descriptor(
  name='Symbols',
  full_name='indexData.Symbols',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rspHeader', full_name='indexData.Symbols.rspHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='symbol', full_name='indexData.Symbols.symbol', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=200,
  serialized_end=266,
)

_RSPHEADER.fields_by_name['ret_type'].enum_type = _RET_TYPE
_JSONRESPONSE.fields_by_name['rspHeader'].message_type = _RSPHEADER
_SYMBOLS.fields_by_name['rspHeader'].message_type = _RSPHEADER
DESCRIPTOR.message_types_by_name['RspHeader'] = _RSPHEADER
DESCRIPTOR.message_types_by_name['JSONRequest'] = _JSONREQUEST
DESCRIPTOR.message_types_by_name['JSONResponse'] = _JSONRESPONSE
DESCRIPTOR.message_types_by_name['Symbols'] = _SYMBOLS
DESCRIPTOR.enum_types_by_name['RET_TYPE'] = _RET_TYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RspHeader = _reflection.GeneratedProtocolMessageType('RspHeader', (_message.Message,), dict(
  DESCRIPTOR = _RSPHEADER,
  __module__ = 'indexData_pb2'
  # @@protoc_insertion_point(class_scope:indexData.RspHeader)
  ))
_sym_db.RegisterMessage(RspHeader)

JSONRequest = _reflection.GeneratedProtocolMessageType('JSONRequest', (_message.Message,), dict(
  DESCRIPTOR = _JSONREQUEST,
  __module__ = 'indexData_pb2'
  # @@protoc_insertion_point(class_scope:indexData.JSONRequest)
  ))
_sym_db.RegisterMessage(JSONRequest)

JSONResponse = _reflection.GeneratedProtocolMessageType('JSONResponse', (_message.Message,), dict(
  DESCRIPTOR = _JSONRESPONSE,
  __module__ = 'indexData_pb2'
  # @@protoc_insertion_point(class_scope:indexData.JSONResponse)
  ))
_sym_db.RegisterMessage(JSONResponse)

Symbols = _reflection.GeneratedProtocolMessageType('Symbols', (_message.Message,), dict(
  DESCRIPTOR = _SYMBOLS,
  __module__ = 'indexData_pb2'
  # @@protoc_insertion_point(class_scope:indexData.Symbols)
  ))
_sym_db.RegisterMessage(Symbols)



_INDEXDATA = _descriptor.ServiceDescriptor(
  name='indexData',
  full_name='indexData.indexData',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=343,
  serialized_end=477,
  methods=[
  _descriptor.MethodDescriptor(
    name='sayHello',
    full_name='indexData.indexData.sayHello',
    index=0,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_JSONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getSymbols',
    full_name='indexData.indexData.getSymbols',
    index=1,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_SYMBOLS,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_INDEXDATA)

DESCRIPTOR.services_by_name['indexData'] = _INDEXDATA

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
import grpc
import sys
sys.path.append("./")
from tradeTools import indexData_pb2,indexData_pb2_grpc
from google.protobuf.json_format import MessageToJson
import json
import better_exceptions
import datetime
import time
import code
import copy
import os
from tradeTools import Decorator
from tradeTools import helpers
import pandas as pd
better_exceptions.hook()

HOST = "localhost"
# HOST = "113.31.119.58"
PORT = 50052

def getIndexSymbols(dateTime = 20100118,indexID = "000300",):
    while 1:
        try:
            with grpc.insecure_channel(f"{HOST}:{PORT}") as channel:
                stub = indexData_pb2_grpc.indexDataStub(channel)

                print("=====【rpc】 getSymbols==========")
                sReq = json.dumps({
                    "indexID":indexID, "dateTime":int(dateTime)})
                response = stub.getSymbols(indexData_pb2.JSONRequest(sReq=sReq))
                jsonRsp = MessageToJson(response)
                jsonRsp = json.loads(jsonRsp)
                return jsonRsp["symbol"]
        except Exception as e:
            print(e)
            time.sleep(3)

class IndexData:
    # todo:akshare 数据不全
    def __init__(self):
        self.indexDfs = {}

    def load(self,indexID = "000300"):
        _realPathFile  = (os.path.realpath(__file__))
        _path = os.path.split(_realPathFile)[0]
        fileName = os.sep.join([_path,r"data", "IndexCons", f"{indexID}指数成分股数据.xlsx"])  #
        df = pd.read_excel(fileName, sheet_name="Sheet", index_col=0,engine="openpyxl")
        df.sort_index(ascending=True, inplace=True)
        df['date'] = pd.to_datetime(df.index, format='%Y-%m-%d')
        df.set_index('date', inplace=True)
        return df

    def getSymbols(self, dateTime:int, indexSymbol="SH.000300"):
        if indexSymbol not in self.indexDfs:
            self.indexDfs[indexSymbol] = self.load(indexID=indexSymbol[-6:])
        indexDF = self.indexDfs[indexSymbol]
        df = indexDF.iloc[indexDF.index <= str(dateTime)]
        symbols = []
        for index, row in df.iterrows():
            symbol = row["代码"][-2:] + "." + row["代码"][:6]
            addOrDel = row[r"纳入/剔除"]
            if addOrDel == "纳入":
                symbols.append(symbol)
            elif addOrDel == "剔除":
                symbols.remove(symbol)
            else:
                print(f"纳入/剔除 列的值错误 #{addOrDel}#")
        symbols.sort()
        return symbols
    
    # @Decorator.loadData(path="data")
    # def qryIndexStockCons(self,symbol = "SH.000300",**kwargs):
    #     import akshare as ak
    #     symbol = symbol[:2].lower()+symbol[-6:]
    #     index_stock_info_df = ak.index_stock_hist(index=symbol)
    #     return index_stock_info_df
    #
    # def getIndexCons(self,dateTime,symbol = "SH.000300"):
    #     cons = self.qryIndexStockCons(symbol = symbol)
    #     cons["tmp"] = cons["in_date"].astype(str)+cons["stock_code"].astype(str)
    #     cons = cons.drop_duplicates('tmp', 'first', ignore_index=True)
    #     cons["in_date"] = pd.to_datetime( cons["in_date"], format='%Y-%m-%d')
    #     cons["out_date"] = pd.to_datetime( cons["out_date"], format='%Y-%m-%d')
    #     into = cons[cons.in_date<=str(dateTime)]["stock_code"].tolist()
    #     print(len(into))
    #     print(into)
    #     ret =[]
    #     for _into in into:
    #         symbol = helpers.code_add_prefix(_into)
    #         ret.append(symbol)
    #     ret.sort()
    #     print(ret)
    
    
    def acquire_date_datas(self, indexSymbol="SH.000300"):
        """构造以日期为 key 的字典"""
        if indexSymbol not in self.indexDfs:
            self.indexDfs[indexSymbol] = self.load(indexID=indexSymbol[-6:])
        
        datas     = {}
        symbols   = []
        last_date = 0
        indexDF   = self.indexDfs[indexSymbol]        
        for index, row in indexDF.iterrows():
            date = index.year * 10000 + index.month * 100 + index.day
            
            if date != last_date or last_date == 0:
                datas[last_date] = symbols
                if len(symbols) > 0: symbols = copy.copy(symbols)
                last_date = date
                            
            symbol   = row["代码"][-2:] + "." + row["代码"][:6]            
            addOrDel = row[r"纳入/剔除"]
            
            if    addOrDel == "纳入": symbols.append(symbol)
            elif  addOrDel == "剔除": symbols.remove(symbol)
            else: print(f"纳入/剔除 列的值错误 #{addOrDel}#")  
        datas[last_date] = symbols              
        return datas
    
if __name__ == '__main__':
    # logging.basicConfig()
    # symbols500 = getIndexSymbols()
    # print(symbols500)
    # symbols300 = getIndexSymbols(indexID="000300")
    # print(symbols300)
    # d = list(set(symbols300)&set(symbols500))
    # print(len(d))
    idxData =IndexData()
    # idxData.qryIndexStockCons(symbol = "SH.000300")
    r = idxData.getSymbols(dateTime = 20211222)
    print(len(r))
    
    r = idxData.acquire_date_datas()
    for k,v in r.items():
        print(k, len(v))